import { isSubscribable, unwrap, isObservable } from "knockout";
import { View, registerView } from "../../../framework/View";
import { ViewModel, extendViewModel, isNullOrUndefined } from "../../../framework/ViewModel";
import { observableProperty, isObservableProperty } from "../../../framework/observable";
import { isInteger, isFloat } from "../../../utilities/common/common";
class InputViewModel extends ViewModel {
    constructor(inputType, options) {
        super(options);
        if (options) {
            if (isObservableProperty(options.value)) {
                this.value = options.value;
            }
            else {
                this.value = observableProperty(options.value);
            }
            this.inputValidator = options.inputValidator;
            this.placeholder = options.placeholder;
            this.valueUpdateMode = options.valueUpdateMode;
            this.type = options.type;
        }
        this.type = inputType;
    }
    value;
    inputValidator;
    placeholder;
    valueUpdateMode = "OnChange";
    type;
}
export class InputTextViewModel extends InputViewModel {
    constructor(options) {
        super("text", options);
    }
}
export class InputCheckboxViewModel extends InputViewModel {
    constructor(options) {
        super("checkbox", options);
    }
}
export class InputFileViewModel extends InputViewModel {
    constructor(options) {
        super("file", options);
    }
}
export class InputIntegerViewModel extends InputViewModel {
    constructor(options) {
        super("integer", options);
    }
}
export class InputFloatViewModel extends InputViewModel {
    constructor(options) {
        super("float", options);
    }
}
export class InputView extends View {
    constructor(viewModel, designTimeOptions) {
        super({
            tagName: "input",
            viewModel: viewModel,
        }, extendViewModel({
            viewType: "Input"
        }, designTimeOptions));
    }
    build() {
        super.build();
        const viewModel = this.viewModel;
        if (!viewModel.valueUpdateMode) {
            viewModel.valueUpdateMode = "OnChange";
        }
        const input = this.element;
        input.type = viewModel.type;
        if (!isNullOrUndefined(viewModel.min)) {
            input.min = viewModel.min;
        }
        if (!isNullOrUndefined(viewModel.max)) {
            input.max = viewModel.max;
        }
        if (!isNullOrUndefined(viewModel.step)) {
            input.step = viewModel.step;
        }
        switch (viewModel.type) {
            case "checkbox":
                input.checked = unwrap(viewModel.value);
                if (isObservable(viewModel.value)) {
                    viewModel.value.subscribe((value) => {
                        input.checked = value;
                    });
                }
                break;
            case "file":
                break;
            default:
                const value = unwrap(viewModel.value);
                if (value) {
                    input.value = value;
                }
                if (isObservable(viewModel.value)) {
                    viewModel.value.subscribe((value) => {
                        input.value = value;
                    });
                }
        }
        const placeholder = unwrap(viewModel.placeholder);
        if (placeholder) {
            input.placeholder = placeholder;
        }
        if (isSubscribable(viewModel.placeholder)) {
            viewModel.placeholder.subscribe((placeholder) => {
                if (placeholder) {
                    input.placeholder = placeholder;
                }
                else {
                    input.removeAttribute("placeholder");
                }
            });
        }
        if (this.viewModel.valueUpdateMode === "OnChange") {
            this.element.addEventListener("change", handleEvent);
        }
        if (this.viewModel.valueUpdateMode === "OnInput" || viewModel.inputValidator) {
            this.element.addEventListener("input", handleEvent);
        }
        async function handleEvent(event) {
            let newValue;
            switch (viewModel.type) {
                case "checkbox":
                    newValue = input.checked;
                    break;
                case "file":
                    newValue = input.files;
                    break;
                default:
                    newValue = input.value;
                    if (newValue + "" === "") {
                        newValue = null;
                    }
            }
            if (viewModel.inputValidator) {
                if (!await viewModel.inputValidator(event, viewModel, newValue)) {
                    input.value = unwrap(viewModel.value);
                    return;
                }
                ;
            }
            if (viewModel.type === "integer") {
                if (isInteger(newValue)) {
                    newValue = parseInt(newValue);
                }
                else {
                    input.value = unwrap(viewModel.value);
                    return;
                }
            }
            if (viewModel.type === "float") {
                if (isFloat(newValue)) {
                    newValue = parseFloat(newValue);
                }
                else {
                    input.value = unwrap(viewModel.value);
                    return;
                }
            }
            if (isObservableProperty(viewModel.value)) {
                if ((event.type === "change" && viewModel.valueUpdateMode === "OnChange") || (event.type === "input" && viewModel.valueUpdateMode === "OnInput")) {
                    viewModel.value.set(newValue);
                }
                else {
                    viewModel.value.set(newValue, false);
                }
            }
        }
    }
}
registerView(InputTextViewModel, InputView);
registerView(InputCheckboxViewModel, InputView);
registerView(InputFileViewModel, InputView);
registerView(InputIntegerViewModel, InputView);
registerView(InputFloatViewModel, InputView);
//# sourceMappingURL=Input.js.map